@@ -905,7 +905,8 @@ sub app_setup {
 	  'convert-config=s',
 
 	  # This aborts option scanning.
-	  'reference|R'		 => sub { $reference++; die("!FINISH!"); },
+	  'reference|R'		 => sub { $reference++;
+					  CORE::die("!FINISH!"); },
 
           ### Standard options ###
 
@@ -1398,6 +1399,23 @@ sub app_usage {
 	$cfg{$_} = $configs{$_} || "no default";
     }
 
+    if ( $::options->{reference} ) {
+	print ${fh} <<EndOfUsage;
+Usage: $0 [ options ] [ file ... ]
+
+Options:
+    --[no]strict        Strict conformance
+    --output=FILE  -o   Saves the output to FILE
+    --about  -A         About ChordPro...
+    --version  -V       Prints version and exits
+    --help  -h          This message
+    --ident             Show identification
+    --verbose           Verbose information. Repeat for more.
+EndOfUsage
+	exit $exit if defined $exit;
+	return;
+    }
+
     ####TODO: weed out for --reference.
     print ${fh} <<EndOfUsage;
 Usage: $0 [ options ] [ file ... ]
